
//	072611

const IS_DBG = false;
const IS_TEST_MODE = false;

var Util = {};

Util.$ = function(id) { return (document.getElementById(id)); };

Util.limit_3 = function(a, b, c)
{
    return a < b ? b : (a > c ? c : a);
}

Util.createKey = function(key)
{
	if (window.widget)
        key = widget.identifier + "-" + key;
	
	return (key);
}

Util.getLocalizedString = function(key)
{
	try { key = localizedStrings[key] || key; } catch (e) { if (IS_DBG) alert(key); return key;}

	return (key);
}

Util.showElement = function(el)
{
	var	e = el;

	if (typeof (el) == 'string')
		e = this.$(el);

	e.style.display = "block";
};

Util.hideElement = function(el)
{
	var	e = el;

	if (typeof (el) == 'string')
		e = this.$(el);

	e.style.display = "none";
};

Util.isShowing = function(el)
{
	var	e = el;

	if (typeof (el) == 'string')
		e = this.$(el);

	return (e.style.display != "none");
};

/**
 *	Return first ancestor element or element itself with a
 *	tag name equal to tagName case-insensitive.
 */

Util.getFirstAncestorOrSelfByTagName = function(element, tagName)
{
	tagName = tagName.toLowerCase();

	do
	{
		if (element.nodeType == Node.ELEMENT_NODE && element.tagName.toLowerCase() == tagName)
			return (element);
	
	} while (element = element.parentNode);
	
	return null;
};

/**
 *	Adds newClassName to list of whitespace-separated class names for node if 
 *	not already present. If newClassName is added, returns true, else false.
 */
Util.addClassName = function(node, newClassName)
{
	var oldClassName = Util.collapseWhiteSpace(Util.trimWhiteSpace(node.className));

	if (oldClassName.indexOf(newClassName) == 0)
		return (false);
	
	var index = oldClassName.indexOf(" " + newClassName);
	
	if (index == -1)
	{
		newClassName = (oldClassName + " " + newClassName);
		node.className = newClassName;
		
		return (true);
	}
	
	return (false);
};

/**
 *	Removes removeClassName to list of whitespace-separated class names for node 
 *	if present. If removeClassName is removed, returns true, else false.
 */
Util.removeClassName = function(node, removeClassName)
{
	if (node && node.className)
	{
		var index = node.className.indexOf(removeClassName);
		
		if (index != -1)
		{
			var len = removeClassName.length;
			node.className = node.className.substring(0,index) + node.className.substring(index+len);
			
			return (true);
		}
	}
	
	return (false);
};

Util.consumeEvent = function(evt)
{
	if (evt)
	{
		evt.stopPropagation();
		evt.preventDefault();
	}
};

Util.getEntireComputedHeight = function(el, pseudoClass)
{
	var style = document.defaultView.getComputedStyle(el, pseudoClass);
	var res = parseInt(style.height);
	
	res += parseInt(style.paddingTop);
	res += parseInt(style.paddingBottom);
	res += parseInt(style.marginTop);
	res += parseInt(style.marginBottom);
	res += parseInt(style.borderTopWidth);
	res += parseInt(style.borderBottomWidth);
	
	return (res);
};


Util.domToString = function(dom) { return ("{id:"+dom.id+", class:"+dom.className+", nodeName:"+dom.nodeName+"}"); }

Util.dumpToString = function(o,depth,newLines,indent)
{
	var retVal = "";

	if (newLines == null || newLines == true)
		newLines = depth - 1;

	if (indent == null)
		indent = 1;

	if (depth == null)
		depth = 1;

	if (typeof o == "string" || typeof o == "number" || typeof o == "boolean" || typeof o == "function")
	{
		if (newLines >= 0)
			for (ii = 0;ii < indent; ii++)
				retVal += "    ";retVal += "'"+o+(newLines >= 0 ?"'\n" : "' ");
	}
	else if (depth > 0)
	{
		for (ii = 0;ii < indent;ii++)
			retVal += "    ";retVal += (newLines > 0 ? "{\n" : "{");

		for (f in o)
		{
			if (newLines > 0)
				for (ii = 0;ii < indent; ii++)	retVal += "    ";
				retVal += f+":"+ this.dumpToString(o[f],depth-1,newLines-1,indent+1);
		}
		
		for (ii = 0;ii < (indent-1); ii++)	retVal += "    ";
		retVal += (newLines >= 0 ? "}\n" : "}");
	}
	else if (newLines >= 0)
		retVal += "\n";
	
	return (retVal);
}

/** Trim all leading and trailing whitespace. */
Util.trimWhiteSpace = function(str)
{
	return str.replace(Util.trimWhiteSpace.regex0, "")
		.replace(Util.trimWhiteSpace.regex1, "");
};

Util.trimWhiteSpace.regex0 = /^\s*/;
Util.trimWhiteSpace.regex1 = /\s*$/;


/** Collapse all runs of whitespace chars in str to a single ' ' space char. */
Util.collapseWhiteSpace = function(str)
{
	return (str.replace(Util.collapseWhiteSpace.regex, " "));
};

Util.collapseWhiteSpace.regex = /\s{2,}/;

/*
function printBackTrace()
{
	var fn = arguments.caller;
	alert("backTrace");
	while (fn != null) {
		alert("    "+fn.toString());
		fn = fn.caller;
	}
}
*/

